unit fMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, StdCtrls, OrbTraceRoute, OrbTraceRouteErrors, ExtCtrls, ComCtrls,
  ScktComp, OrbTraceRouteThread, winsock;

type
  TfrmMain = class(TForm)
    ThreadMonitorTimer: TTimer;
    Panel3: TPanel;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    Label2: TLabel;
    Label14: TLabel;
    Label17: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label1: TLabel;
    cboTraceRouteNameOrAddress: TComboBox;
    Button2: TButton;
    cboTraceRoutePriority: TComboBox;
    edtTraceRouteTimeout: TEdit;
    edtTraceRouteTTL: TEdit;
    edtTraceRouteDataSize: TEdit;
    Button5: TButton;
    edtMaxThreads: TEdit;
    ResultList: TMemo;
    ListView1: TListView;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    cboTraceRouteNameOrAddressFast: TComboBox;
    Button1: TButton;
    cboTraceRoutePriorityFast: TComboBox;
    edtTraceRouteTimeoutFast: TEdit;
    edtTraceRouteTTLFast: TEdit;
    edtTraceRouteDataSizeFast: TEdit;
    Button3: TButton;
    Edit4: TEdit;
    Memo1: TMemo;
    ListView2: TListView;
    StatusBar2: TStatusBar;
    StatusBar1: TStatusBar;
    Timer1: TTimer;
    edtTraceRouteTag: TEdit;
    Label10: TLabel;
    OrbTraceRouteFast: TOrbTraceRoute;
    OrbTraceRouteSlow: TOrbTraceRoute;
    edtTraceRouteTagFast: TEdit;
    Label11: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure ThreadMonitorTimerTimer(Sender: TObject);
    procedure OrbTraceRouteSlowBeforeResolveEvent(var AsIP, AsName: String;
      AsTag: String; var AbHandled: Boolean);
    procedure OrbTraceRouteSlowErrorEvent(OrbError: TOrbError);
    procedure OrbTraceRouteSlowNodeEvent(NodeData: TNodeData);
    procedure OrbTraceRouteFastBeforeResolveEvent(var AsIP, AsName: String;
      AsTag: String; var AbHandled: Boolean);
    procedure OrbTraceRouteFastErrorEvent(OrbError: TOrbError);
    procedure OrbTraceRouteFastNodeEvent(NodeData: TNodeData);
    procedure Button1Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    Cache: TStringList;
    function ClearCache: LongInt;
    function GetThreadTypeFromString(AsThreadString: string): TThreadPriority;
  public
  end;

Type
  TCacheItem = Class
    Name: string;
    IP: string;
end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

//form create
procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Cache := TStringList.Create;
  Cache.Sorted := True;
  Cache.Duplicates := dupIgnore;
  cboTraceRouteNameOrAddress.ItemIndex := 0;
  cboTraceRoutePriority.ItemIndex := 3;
  cboTraceRouteNameOrAddressFast.ItemIndex := 0;
  cboTraceRoutePriorityFast.ItemIndex := 3;
end;

//close form
procedure TfrmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  ClearCache;
  Cache.Free;
end;

//clear the cache - the cache is shared between slow and fast demo tabs
function TfrmMain.ClearCache: LongInt;
var
iCounter: LongInt;
begin
  Result := Cache.Count;
  For ICounter := 0 to Cache.Count - 1 do
  begin
    (Cache.Objects[iCounter] as TCacheItem).Free;
  end;
  Cache.Clear;
end;

//get the thread priority
function TfrmMain.GetThreadTypeFromString(AsThreadString: string):
  TThreadPriority;
begin
  Result := tpNormal;
  if AsThreadString = 'tbIdle' then Result := tpIdle;
  if AsThreadString = 'tpLowest' then Result := tpLowest;
  if AsThreadString = 'tpLower' then Result := tpLower;
  if AsThreadString = 'tpNormal' then Result := tpNormal;
  if AsThreadString = 'tpHigher' then Result := tpHigher;
  if AsThreadString = 'tpHighest' then Result := tpHighest;
  if AsThreadString = 'tpTimeCritical' then Result := tpTimeCritical;
end;

//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//SLOW
//status monitor
procedure TfrmMain.ThreadMonitorTimerTimer(Sender: TObject);
begin
  StatusBar1.Panels[1].Text := 'Active Threads: '
    + IntToStr(OrbTraceRouteSlow.ThreadCount);
  If OrbTraceRouteSlow.Active then
    StatusBar1.Panels[0].Text := 'Status: Tracing...'
  else
    StatusBar1.Panels[0].Text := 'Status: Idle';
end;

//traceroute
procedure TfrmMain.Button2Click(Sender: TObject);
begin
  ResultList.Lines.Clear;
  ListView1.Items.Clear;
  OrbTraceRouteSlow.Cancel;
  OrbTraceRouteSlow.MaxThreads := StrToInt(Trim(edtMaxThreads.Text));
  OrbTraceRouteSlow.Fast := False;
  If not OrbTraceRouteSlow.Trace(
    Trim(cboTraceRouteNameOrAddress.Text),
    Trim(edtTraceRouteTag.Text),
    StrToInt(edtTraceRouteTTL.Text),
    StrToInt(edtTraceRouteTimeout.Text),
    StrToInt(edtTraceRouteDataSize.Text),
    GetThreadTypeFromString(cboTraceRouteNameOrAddress.Items[
      cboTraceRouteNameOrAddress.ItemIndex])) then
    ResultList.Lines.Add('>> Trace Ignored due to Threadshortage.')
  else
   ResultList.Lines.Add('>> Tracing ' + cboTraceRouteNameOrAddress.Items[
      cboTraceRouteNameOrAddress.ItemIndex]);
end;

//cancel all traceroutes
procedure TfrmMain.Button5Click(Sender: TObject);
var
iTraceRoutes: LongInt;
begin
  iTraceRoutes := OrbTraceRouteSlow.Cancel;
  ResultList.Lines.Add('>> Cancelled all TraceRoutes (' + IntToStr(iTraceRoutes)
    + ' total)');
end;

//before resolve event
procedure TfrmMain.OrbTraceRouteSlowBeforeResolveEvent(var AsIP,
  AsName: String; AsTag: String; var AbHandled: Boolean);
var
sTemp1: string;
iIndex: LongInt;
begin
  //this event can be used to try to resolve the ip to a name by using a
  //cache that you create and manage.  In this example, the cache is not
  //persistant and will be destroyed when the app is terminated.
  //A good idea would be to save the cache to a textfile or something when
  //exiting the app, and loading it up into the cache on app start.
  If (AsIP <> '') and (AsName = '') then
  begin
    sTemp1 := LowerCase(AsIP);
    iIndex := Cache.IndexOf(sTemp1);
    If iIndex = -1 then
    begin
      //this ip is not in the cache
      AbHandled := False;
    end
    else
    begin
      //item found in cache
      AsName := (Cache.Objects[iIndex] as TCacheItem).Name;
      AbHandled := True;
      ResultList.Lines.Add('Cache Hit ' + AsName + ' resolved from  ' +
        AsIP);
    end;
  end;
end;

//error event
procedure TfrmMain.OrbTraceRouteSlowErrorEvent(OrbError: TOrbError);
begin
  //error
  ResultList.lines.add('<< TraceRoute Error: ' + IntToStr(OrbError.Code) + ','
    + OrbError.Description + ',' + OrbError.Verbose);
end;

//node event
procedure TfrmMain.OrbTraceRouteSlowNodeEvent(NodeData: TNodeData);
var
CacheItem: TCacheItem;
ListItem: TListItem;
sTemp1: string;
iIndex: LongInt;
begin
  ListItem := ListView1.Items.Add;
  sTemp1 := '00' + IntToStr(NodeData.Node);
  sTemp1 := Copy(sTemp1, Length(sTemp1) - 1, 2);
  ListItem.Caption := sTemp1;
  If NodeData.IP = '*' then ListItem.SubItems.Add('*')
  else ListItem.SubItems.Add(IntToStr(NodeData.ElapsedTime));
  ListItem.SubItems.Add(NodeData.IP);
  ListItem.SubItems.Add(NodeData.Name);
  //add to cache if valid ip and name
  If (NodeData.IP <> '*') and (NodeData.Name <> '*') then
  sTemp1 := LowerCase(NodeData.IP);
  iIndex := Cache.IndexOf(sTemp1);
  If iIndex = -1 then
  begin
    CacheItem := TCacheItem.Create;
    CacheItem.IP := NodeData.IP;
    CacheItem.Name := NodeData.Name;
    Cache.AddObject(LowerCase(NodeData.IP), CacheItem);
  end;
end;






//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//FAST
//status monitor
procedure TfrmMain.Timer1Timer(Sender: TObject);
begin
  StatusBar2.Panels[1].Text := 'Active Threads: '
    + IntToStr(OrbTraceRouteFast.ThreadCount);
  If OrbTraceRouteFast.Active then
    StatusBar2.Panels[0].Text := 'Status: Tracing...'
  else
    StatusBar2.Panels[0].Text := 'Status: Idle';
end;

//traceroute
procedure TfrmMain.Button1Click(Sender: TObject);
begin
  ResultList.Lines.Clear;
  ListView2.Items.Clear;
  OrbTraceRouteFast.Cancel;
  OrbTraceRouteFast.MaxThreads := StrToInt(Trim(edtMaxThreads.Text));
  OrbTraceRouteFast.Fast := True;
  If not OrbTraceRouteFast.Trace(
    Trim(cboTraceRouteNameOrAddressFast.Text),
    Trim(edtTraceRouteTagFast.Text),
    StrToInt(edtTraceRouteTTLFast.Text),
    StrToInt(edtTraceRouteTimeoutFast.Text),
    StrToInt(edtTraceRouteDataSizeFast.Text),
    GetThreadTypeFromString(cboTraceRouteNameOrAddressFast.Items[
      cboTraceRouteNameOrAddressFast.ItemIndex])) then
    Memo1.Lines.Add('>> Trace Ignored due to Threadshortage.')
   else
   Memo1.Lines.Add('>> Tracing ' + cboTraceRouteNameOrAddressFast.Items[
      cboTraceRouteNameOrAddressFast.ItemIndex]);
end;

//cancel
procedure TfrmMain.Button3Click(Sender: TObject);
var
iTraceRoutes: LongInt;
begin
  iTraceRoutes := OrbTraceRouteFast.Cancel;
  Memo1.Lines.Add('>> Cancelled all TraceRoutes (' + IntToStr(iTraceRoutes)
    + ' total)');
end;

//before resolve event
procedure TfrmMain.OrbTraceRouteFastBeforeResolveEvent(var AsIP,
  AsName: String; AsTag: String; var AbHandled: Boolean);
var
sTemp1: string;
iIndex: LongInt;
begin
  //this event can be used to try to resolve the ip to a name by using a
  //cache that you create and manage.  In this example, the cache is not
  //persistant and will be destroyed when the app is terminated.
  //A good idea would be to save the cache to a textfile or something when
  //exiting the app, and loading it up into the cache on app start.
  If (AsIP <> '') and (AsName = '') then
  begin
    sTemp1 := LowerCase(AsIP);
    iIndex := Cache.IndexOf(sTemp1);
    If iIndex = -1 then
    begin
      //this ip is not in the cache
      AbHandled := False;
    end
    else
    begin
      //item found in cache
      AsName := (Cache.Objects[iIndex] as TCacheItem).Name;
      AbHandled := True;
      Memo1.Lines.Add('Cache Hit ' + AsName + ' resolved from  ' +
        AsIP);
    end;
  end;
end;

//error event
procedure TfrmMain.OrbTraceRouteFastErrorEvent(OrbError: TOrbError);
begin
  //error
  Memo1.lines.add('<< TraceRoute Error: ' + IntToStr(OrbError.Code) + ','
    + OrbError.Description + ',' + OrbError.Verbose);
end;

//node event
procedure TfrmMain.OrbTraceRouteFastNodeEvent(NodeData: TNodeData);
var
CacheItem: TCacheItem;
ListItem: TListItem;
sTemp1: string;
iLowest, iIndex: LongInt;
begin
  ListItem := ListView2.Items.Add;
  sTemp1 := '00' + IntToStr(NodeData.Node);
  sTemp1 := Copy(sTemp1, Length(sTemp1) - 1, 2);
  ListItem.Caption := sTemp1;
  If NodeData.IP = '*' then ListItem.SubItems.Add('*')
  else ListItem.SubItems.Add(IntToStr(NodeData.ElapsedTime));
  ListItem.SubItems.Add(NodeData.IP);
  ListItem.SubItems.Add(NodeData.Name);
  //add to cache if valid ip and name
  If (NodeData.IP <> '*') and (NodeData.Name <> '*') then
  sTemp1 := LowerCase(NodeData.IP);
  iIndex := Cache.IndexOf(sTemp1);
  If iIndex = -1 then
  begin
    CacheItem := TCacheItem.Create;
    CacheItem.IP := NodeData.IP;
    CacheItem.Name := NodeData.Name;
    Cache.AddObject(LowerCase(NodeData.IP), CacheItem);
  end;
  //get rid of duplicate end nodes.  This is the inconvenient part of doing
  //a fast trace
  iLowest := StrToInt(edtTraceRouteTTLFast.Text);
  For iIndex := 0 to ListView2.Items.Count - 1 do
  begin
    If ListView2.Items[iIndex].SubItems[1] = NodeData.IP then
    begin
      If iIndex < iLowest then
        iLowest := iIndex;
    end;
  end;
  For iIndex := 0 to ListView2.Items.Count - 1 do
  begin
    If ListView2.Items[iIndex].SubItems[1] = NodeData.IP then
    begin
      If iIndex <> iLowest then
        ListView2.Items.Delete(iIndex);
    end;
  end;
end;


end.
